﻿using AgileUtility.Printer.Utils;
using MahApps.Metro.Controls;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Printing;
using System.IO;
using System.Windows;
using System.Xml.Linq;
using ThoughtWorks.QRCode.Codec;

namespace AgileUtility.Printer
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : MetroWindow
    {
        public MainWindow()
        {
            InitializeComponent();

            this.Loaded += MainWindow_Loaded;
        }



        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            txtTicketTemp.Text = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "PrintTemplate", "TicketTemp.frx");
            txtBillTemp.Text = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "PrintTemplate", "BillTemp.frx");
            cbxPrinter.ItemsSource = this.GetLocalPrinters();
        }



        private void BtnTicketDesign_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(txtTicketTemp.Text))
            {
                MessageBox.Show("门票模板不能为空");
                return;
            }                

            var data = this.CreateTicketData();
            if (!data.Item1)
            {
                MessageBox.Show("模拟数据生成错误");
                return;
            }

            var tuple = PrintHelper.Design(txtTicketTemp.Text, data.Item2, data.Item3);
            if (!tuple.Item1)
            {
                MessageBox.Show($"打开设计器失败：{tuple.Item2}");
            }
        }

        private void BtnTicketPrint_Click(object sender, RoutedEventArgs e)
        {
            if (cbxPrinter.SelectedValue == null || string.IsNullOrEmpty(cbxPrinter.SelectedValue.ToString()))
            {
                MessageBox.Show("请选择打印机");
                return;
            }

            if (string.IsNullOrEmpty(txtTicketTemp.Text))
            {
                MessageBox.Show("门票模板不能为空");
                return;
            }

            var data = this.CreateTicketData();
            if (!data.Item1)
            {
                MessageBox.Show("模拟数据生成错误");
                return;
            }


            var tuple = PrintHelper.Print(cbxPrinter.SelectedValue.ToString(), txtTicketTemp.Text, data.Item2, data.Item3);
            if (!tuple.Item1)
            {
                MessageBox.Show($"打印失败：{tuple.Item2}");
            }
        }

        private void BtnBillDesign_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(txtBillTemp.Text))
            {
                MessageBox.Show("门票模板不能为空");
                return;
            }

            var data = this.CreateBillData();
            if (!data.Item1)
            {
                MessageBox.Show("模拟数据生成错误");
                return;
            }

            var tuple = PrintHelper.Design(txtBillTemp.Text, data.Item2, data.Item3);
            if (!tuple.Item1)
            {
                MessageBox.Show($"打开设计器失败：{tuple.Item2}");
            }
        }

        private void BtnBillPrint_Click(object sender, RoutedEventArgs e)
        {
            if (cbxPrinter.SelectedValue == null || string.IsNullOrEmpty(cbxPrinter.SelectedValue.ToString()))
            {
                MessageBox.Show("请选择打印机");
                return;
            }

            if (string.IsNullOrEmpty(txtBillTemp.Text))
            {
                MessageBox.Show("门票模板不能为空");
                return;
            }

            var data = this.CreateBillData();
            if (!data.Item1)
            {
                MessageBox.Show("模拟数据生成错误");
                return;
            }

            var tuple = PrintHelper.Print(cbxPrinter.SelectedValue.ToString(), txtBillTemp.Text, data.Item2, data.Item3);
            if (!tuple.Item1)
            {
                MessageBox.Show($"打印失败：{tuple.Item2}");
            }
        }

        /// <summary>
        /// 组装门票打印模拟数据
        /// </summary>
        /// <returns></returns>
        private Tuple<bool, Dictionary<string, object>, DataSet> CreateTicketData()
        {
            //注意事项
            //文本内容放入字段中，传入就可以
            //图片内容有两种方案
            //1：使用模板的Barcode标签，这个支持标准qr码，条码等等多种类型，字典或者数据源传入文本值，自动会显示qr码条码图片
            //2：使用模板的Picture标签，使用这个就是要程序生成好图片，再把图片显示，这里就不仅仅限制于二维码条码了，各种想显示的图片都可以，但需要把图片流放到数据源中传入
            Dictionary<string, object> dic = new Dictionary<string, object>();
            dic.Add("ticketModelName", "XXX景点门票");
            dic.Add("ticketModelKind", "成人票");
            dic.Add("ticketModelPrice", "￥100");
            //字典中传入二维码的文本，fastreport提供了生成qr码以及各种条码
            dic.Add("barcode", "ET2018000000000000001");

            //如果需要，组装dataset数据源，这里以传入二维码图片为例
            DataTable dtImage = new DataTable("dtBarcode");
            dtImage.Columns.Add("barcode", typeof(Byte[]));

            DataSet dsFrx = new DataSet();
            dsFrx.Tables.Add(dtImage);

            //1、把二维码码文本生成图片  这个有很多第三方库可以支持  我这里用 ThoughtWorks.QRCode
            QRCodeEncoder qrCodeEncoder = new QRCodeEncoder();
            qrCodeEncoder.QRCodeEncodeMode = QRCodeEncoder.ENCODE_MODE.BYTE;
            qrCodeEncoder.QRCodeScale = 4;
            qrCodeEncoder.QRCodeVersion = 4;
            qrCodeEncoder.QRCodeErrorCorrect = QRCodeEncoder.ERROR_CORRECTION.M;

            using (Image image = qrCodeEncoder.Encode("ET2018000000000000001"))
            {
                //2、生成的图片本地可以做个备份记录，也可以不需要直接将image转byte[]传人数据源就可以
                if (!Directory.Exists(Path.Combine(Directory.GetCurrentDirectory(), "BarCode")))
                {
                    Directory.CreateDirectory(Path.Combine(System.IO.Directory.GetCurrentDirectory(), "BarCode"));
                }
                string filename = DateTime.Now.ToString("yyyymmddhhmmssfff").ToString() + ".jpg";
                string filepath = Path.Combine(System.IO.Directory.GetCurrentDirectory(), "BarCode", filename);
                using (FileStream fs = new FileStream(filepath, FileMode.OpenOrCreate, FileAccess.Write))
                {
                    image.Save(fs, ImageFormat.Jpeg);
                }

                //3、将image转byte[]传人数据源  注意图片传入的是字节数组byte[]  不是文本也不是图片路径！！！
                dtImage.Rows.Add(ImageToBytes(image, ImageFormat.Jpeg));
            }
            return new Tuple<bool, Dictionary<string, object>, DataSet>(true, dic, dsFrx);
        }

        /// <summary>
        /// 组装小票打印模拟数据
        /// </summary>
        /// <returns></returns>
        private Tuple<bool, Dictionary<string, object>, DataSet> CreateBillData()
        {
            //注意事项
            //小票打印和门票一样，主要的区别是小票动态数据会变化，小票的长度也会动态改变
            //这里主要演示下  动态数据源  为了动态拉伸，除了传入数据源，在模板上面code部分需要加代码
            Dictionary<string, object> dic = new Dictionary<string, object>();
            dic.Add("billNo", "2018111100002222");
            dic.Add("optorName", "管理员");

            //组装dataset数据源
            DataTable dtDetail = new DataTable("dtDetail");
            dtDetail.Columns.Add("GOODSCODE");
            dtDetail.Columns.Add("GOODSNAME");
            dtDetail.Columns.Add("GOODSPRICE");
            dtDetail.Columns.Add("GOODSCOUNT");
            dtDetail.Columns.Add("PAYSUM");

            //加10种商品
            for (int i = 1; i <= 10; i++)
            {
                dtDetail.Rows.Add("10000" + 1, "测试商品" + i, 10.00m, 5, 50.00m);
            }


            DataSet dsFrx = new DataSet();
            dsFrx.Tables.Add(dtDetail);

            return new Tuple<bool, Dictionary<string, object>, DataSet>(true, dic, dsFrx);
        }

        private List<XElement> GetLocalPrinters()
        {
            List<XElement> list = new List<XElement>();
            try
            {
                foreach (string fPrinterName in PrinterSettings.InstalledPrinters)
                {
                    var xePrintName = new XElement("Item", new XElement("OPTIONNAME", fPrinterName.Trim()), new XElement("OPTIONCODE", fPrinterName.Trim()));

                    if (!list.Contains(xePrintName))
                        list.Add(xePrintName);
                }
            }
            catch (Exception ex)
            {

            }

            return list;
        }


        /// <summary>
        /// 将图片Image转换成Byte[]
        /// </summary>
        /// <param name="Image">image对象</param>
        /// <param name="imageFormat">后缀名</param>
        /// <returns></returns>
        private static byte[] ImageToBytes(Image Image, System.Drawing.Imaging.ImageFormat imageFormat)
        {
            if (Image == null) { return null; }
            byte[] data = null;
            using (MemoryStream ms = new MemoryStream())
            {
                using (Bitmap bitmap = new Bitmap(Image))
                {
                    bitmap.Save(ms, imageFormat);
                    ms.Position = 0;
                    data = new byte[ms.Length];
                    ms.Read(data, 0, Convert.ToInt32(ms.Length));
                    ms.Flush();
                }
            }
            return data;
        }
    }
}
